import React, { useState, useCallback, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { csv, caleSqrt, scaleLinear, scaleOrdinal, scaleTime, max, min, timeFormat, extent, pointRadial, nest } from 'd3';
import { useData } from './useData';
import { Marks } from './Marks';
import { ColorLegend } from './ColorLegend';

const width = 960;
const height = 500;
const margin = { top: 50, right: 180, bottom: 10, left: 30 };
const angleAxisOffset = 40;
const fullCircle = 2 * Math.PI;
const colorLegendLeftPadding = 40;
const colorLegendTopPadding = 30;
const fadeOpacity = 0.1;

const App = () => {
  const data = useData();
  const [hoveredValue, setHoveredValue] = useState(null);

  if (!data) {
    return <pre>Loading...</pre>;
  }
  
  // console.log(data[0]);
  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;
  const innerRadius = innerHeight/5;
  const outerRadius = innerHeight/2.5;
  const title = 'On-time Performance of top-5 busiest airports in USA'

  const xValue = d => d.DATE;
  const xAxisLabel = 'DATE';
  const angleScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, fullCircle]);

  const yValue = d => d.DEP_DELAY;
  const yAxisLabel = 'Avg_Delay(mins)';
  const radiusScale = scaleLinear()
    .domain([0, max(data, yValue)])
    .range([innerRadius, outerRadius])
    .nice();
  
  const colorValue = d => d.AIRPORT
  const colorScale = scaleOrdinal()
    .domain(data.map(colorValue))
    .range(['#EF7F67', '#B1A194', '#E5BB6F', '#99BBA2', '#938FA8'])

  
  const colorLegendLabel = 'Airports';
  const filteredData = data.filter(d => hoveredValue === colorValue(d));
  
  const xAxisTickFormat = timeFormat('%b');
  
  const nested = nest()
    .key(colorValue)
    .entries(data)

  return (
    
    <svg width={width} height={height}>
      <text y={margin.top/2} x={width/2} class='title'>{title}</text>
      <g transform={`translate(${innerWidth/2 + margin.left}, ${innerHeight /2 + margin.top})`} >
        {angleScale.ticks().map(tickValue => ( 
          <g className="tick" key={tickValue} transform={`translate(${pointRadial(angleScale(tickValue), outerRadius)[0]}, ${pointRadial(angleScale(tickValue), outerRadius)[1]})`}>    
            <line 
              x1={-pointRadial(angleScale(tickValue), innerRadius - angleAxisOffset)[0]}
              y1={-pointRadial(angleScale(tickValue), innerRadius - angleAxisOffset)[1]}
              stroke="grey"
            />           
            <text>{xAxisTickFormat(tickValue)}</text>
          </g>
        ))}
        
        <g transform={`translate(${innerWidth/2 }, ${-innerHeight/2})`}>
          <ColorLegend 
            tickSpacing={20}
            tickWidth={30}
            tickHeight={3}
            tickTextOffset={colorLegendLeftPadding}
            tickYOffset={20+colorLegendTopPadding}
            colorScale={colorScale}
            onHover={setHoveredValue}
            hoveredValue={hoveredValue}
            fadeOpacity={fadeOpacity}
            />
          
          <text
            x={50}
            y={20}
            className="axis-label"
            textAnchor="middle"
          >
            {colorLegendLabel}
          </text>
        </g>
        
       <g opacity={hoveredValue ? fadeOpacity : 1}>
          <Marks
            data={nested[0].values}
            angleScale={angleScale}
            radiusScale={radiusScale}
            xValue={xValue}
            yValue={yValue}
            colorValue={colorValue}
            colorScale={colorScale}
            tooltipFormat={xAxisTickFormat}
            hoveredValue={nested[0].key}
          />
          <Marks
            data={nested[1].values}
            angleScale={angleScale}
            radiusScale={radiusScale}
            xValue={xValue}
            yValue={yValue}
            colorValue={colorValue}
            colorScale={colorScale}
            tooltipFormat={xAxisTickFormat}
            hoveredValue={nested[1].key}
          />
          <Marks
            data={nested[2].values}
            angleScale={angleScale}
            radiusScale={radiusScale}
            xValue={xValue}
            yValue={yValue}
            colorValue={colorValue}
            colorScale={colorScale}
            tooltipFormat={xAxisTickFormat}
            hoveredValue={nested[2].key}
          />
        <Marks
            data={nested[3].values}
            angleScale={angleScale}
            radiusScale={radiusScale}
            xValue={xValue}
            yValue={yValue}
            colorValue={colorValue}
            colorScale={colorScale}
            tooltipFormat={xAxisTickFormat}
            hoveredValue={nested[3].key}
          />
          <Marks
            data={nested[4].values}
            angleScale={angleScale}
            radiusScale={radiusScale}
            xValue={xValue}
            yValue={yValue}
            colorValue={colorValue}
            colorScale={colorScale}
            tooltipFormat={xAxisTickFormat}
            hoveredValue={nested[4].key}
          />
        </g> 
        
        <Marks
          data={filteredData}
          angleScale={angleScale}
          radiusScale={radiusScale}
          xValue={xValue}
          yValue={yValue}
          colorValue={colorValue}
          colorScale={colorScale}
          tooltipFormat={xAxisTickFormat}
          hoveredValue={hoveredValue}
        />
      </g>
    </svg>
  );
};
const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
